import java.io.IOException;

import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Level;
import org.jetel.component.AbstractGenericTransform;
import org.jetel.data.DataRecord;
import org.jetel.exception.ComponentNotReadyException;
import org.jetel.exception.ConfigurationStatus;
import org.jetel.exception.JetelRuntimeException;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.impl.recovery.AutorecoveringChannel;

import sun.util.logging.resources.logging;
/**
 * RabbitMqReaderAck is a component that invokes Channel.basicAck() for each successfully processed message. 
 * We use messageId that is an identifier of a message read through the opened channel.
 * 
 * This component uses the same RabbitMQ Channel (and therefore the same Connection) that was opened by the message reader (RabbitMqReader.java). Therefore, it cannot
 *  work without this component as this component does not open any connection. Connectioned used by this component is received from a dictionary entry into which the message 
 *  was written using the RabbitMqreader.java. 
 * 
 * Connection is completely handled by the previous component - will be automatically closed as the end of the phase.
 *  
 *  !!! WHY REFLECTION API !!!
 * Requires usage of Java reflection API, as by default each component has its own classloader.
 * The default behavior might be overridden on the sandbox or graph level, but then it is applied to all components in the graph,
 *  and therefore might be a risk in concurrent processing of the components.
 */
public class RabbitMqReaderAck extends AbstractGenericTransform {

	private static final String MESSAGE_ID_FIELD_NAME = "messageId";
	private static final String RMQ_CHANNEL_DICTIONARY = "rabbitMqChannel";
	
	@Override
	public void execute() {
		
		DataRecord record;
		
		Class[] args = new Class[2];
		args[0] = Long.TYPE;
		args[1] = Boolean.TYPE;
		
		Method m;
		long messageId;
		try {
			Class cls = graph.getDictionary().getEntry(RMQ_CHANNEL_DICTIONARY).getValue().getClass();
			while ((record = readRecordFromPort(0)) != null) {
				messageId = (long) record.getField(MESSAGE_ID_FIELD_NAME).getValue();
				m = cls.getMethod("basicAck", args);
				m.invoke(graph.getDictionary().getEntry(RMQ_CHANNEL_DICTIONARY).getValue(), messageId, false);
			}
		} catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			throw new JetelRuntimeException("Delivery of the message couldn't be acknowledged.", e);
		} 
	}

	@Override
	public ConfigurationStatus checkConfig(ConfigurationStatus status) {
		super.checkConfig(status);

		/** This way you can check connected edges and their metadata. */
		/*
		if (getComponent().getInPorts().size() < 1) {
			status.add("Input port must be connected!", Severity.ERROR, getComponent(), Priority.NORMAL);
			return status;
		}

		DataRecordMetadata inMetadata = getComponent().getInputPort(0).getMetadata();
		if (inMetadata == null) {
			status.add("Metadata on input port not specified!", Severity.ERROR, getComponent(), Priority.NORMAL);
			return status;
		}

		if (inMetadata.getFieldPosition("myMetadataFieldName") == -1) {
			status.add("Incompatible input metadata!", Severity.ERROR, getComponent(), Priority.NORMAL);
		}
		*/
		return status;
	}

	@Override
	public void init() {
		super.init();
	}

	@Override
	public void preExecute() throws ComponentNotReadyException {
		super.preExecute();
	}

	@Override
	public void postExecute() throws ComponentNotReadyException {
		super.postExecute();
	}
}
